import sys

import json
import numpy as np
from obspy.core import Stream, Stats

from ..adjusted import AdjustedMatrix
from .Algorithm import Algorithm


class AdjustedAlgorithm(Algorithm):
    """Algorithm that converts from one geomagnetic coordinate system to a
    related geographic coordinate system, by using transformations generated
    from absolute, baseline measurements.
    """

    def __init__(
        self,
        matrix: AdjustedMatrix = None,
        statefile=None,
        data_type=None,
        location=None,
        inchannels=None,
        outchannels=None,
    ):
        Algorithm.__init__(
            self,
            inchannels=inchannels,
            outchannels=outchannels,
        )
        # state variables
        self.matrix = matrix
        self.statefile = statefile
        self.data_type = data_type
        self.location = location
        # initialize state from statefile or inchannels;
        # override both with matrix, and don't call load_state
        if (statefile or inchannels) and (matrix is None):
            self.load_state()

    def load_state(self):
        """Load algorithm state from a file.
        File name is self.statefile.
        """
        if self.statefile is None:
            # if no statefile provided, default to identity Adjusted matrix
            # (requires input channels to be specified...is this desirable?)
            matrix_size = len([c for c in self.get_input_channels() if c != "F"]) + 1
            matrix = np.eye(matrix_size).tolist()
            self.matrix = AdjustedMatrix(matrix=matrix)
            return
        try:
            with open(self.statefile, "r") as f:
                data = f.read()
                data = json.loads(data)
        except IOError as err:
            raise FileNotFoundError("statefile not found")
        if "pier_correction" in data:
            self.matrix = AdjustedMatrix(**data)
        elif "PC" in data:
            # read data from legacy format
            matrix_size = int(
                # assumes matrix is square
                np.sqrt(len([k for k in data.keys() if k.startswith("M")]))
            )
            matrix = np.eye(matrix_size).tolist()
            for row in range(matrix_size):
                for col in range(matrix_size):
                    matrix[row][col] = np.float64(data[f"M{row+1}{col+1}"])
            pier_correction = np.float64(data["PC"])
            self.matrix = AdjustedMatrix(matrix=matrix, pier_correction=pier_correction)
        else:
            raise ValueError("pier correction not found in statefile")

    def save_state(self):
        """Save algorithm state to a file.
        File name is self.statefile.
        """
        if self.statefile is None:
            return
        json_string = self.matrix.json()
        json_dict = json.loads(json_string)
        with open(self.statefile, "w") as f:
            f.write(json.dumps(json_dict))

    def create_trace(self, channel, stats, data):
        """Utility to create a new trace object.

        Parameters
        ----------
        channel : str
            channel name.
        stats : obspy.core.Stats
            channel metadata to clone.
        data : numpy.array
            channel data.

        Returns
        -------
        obspy.core.Trace
            trace containing data and metadata.
        """
        stats = Stats(stats)
        if self.data_type is None:
            stats.data_type = "adjusted"
        else:
            stats.data_type = self.data_type
        if self.data_type is None:
            stats.location = "A0"
        else:
            stats.location = self.location

        trace = super(AdjustedAlgorithm, self).create_trace(channel, stats, data)
        return trace

    def process(self, stream):
        """Run algorithm for a stream.
        Processes all traces in the stream.
        Parameters
        ----------
        stream : obspy.core.Stream
            stream of data to process
        Returns
        -------
        out : obspy.core.Stream
            stream containing 1 trace per original trace. (h, e, z)->(X, Y, Z)
        """

        out = None
        inchannels = self.get_input_channels() or [
            trace.stats.channel for trace in stream
        ]
        outchannels = self.get_output_channels() or inchannels

        adjusted = self.matrix.process(
            stream,
            inchannels=inchannels,
            outchannels=outchannels,
        )

        # if F in inchannels/outchannels is not in the last position,
        # still return it as the last element the output Stream
        inchannels_noF = [c for c in inchannels if c != "F"]
        outchannels_noF = [c for c in outchannels if c != "F"]

        out_noF = Stream(
            [
                (
                    self.create_trace(
                        outchannels_noF[i],
                        (
                            stream.select(channel=inchannels_noF[i])[0].stats
                            if i < len(inchannels_noF)
                            else stream.select(channel=inchannels_noF[-1])[0].stats
                        ),
                        adjusted[i],
                    )
                )
                for i in range(len(outchannels_noF))
            ]
        )
        out_F = Stream(
            self.create_trace("F", stream.select(channel="F")[0].stats, adjusted[-1])
            if stream.select(channel="F")
            else []
        )

        return out_noF + out_F

    def can_produce_data(self, starttime, endtime, stream):
        """Can Produce data
        Parameters
        ----------
        starttime: UTCDateTime
            start time of requested output
        end : UTCDateTime
            end time of requested output
        stream: obspy.core.Stream
            The input stream we want to make certain has data for the algorithm
        """

        channels = self.get_input_channels()

        for c in channels:
            # non-F channels require valid data for matrix transform
            if c != "F" and not (
                super().can_produce_data(starttime, endtime, stream.select(channel=c))
            ):
                return False
        if "F" not in channels:
            # only matrix transform desired if user didn't pass F
            return True
        elif super().can_produce_data(starttime, endtime, stream.select(channel="F")):
            # F adjustment desired and possible; matrix transform possible
            return True
        # return false if F or remaining channels cannot produce data
        return False

    @classmethod
    def add_arguments(cls, parser):
        """Add command line arguments to argparse parser.
        Parameters
        ----------
        parser: ArgumentParser
            command line argument parser
        """

        parser.add_argument(
            "--adjusted-statefile",
            default=None,
            help="File to store state between calls to algorithm",
        )

    def configure(self, arguments):
        """Configure algorithm using comand line arguments.
        Parameters
        ----------
        arguments: Namespace
            parsed command line arguments
        """
        Algorithm.configure(self, arguments)
        self.statefile = arguments.adjusted_statefile
        self.load_state()
