from typing import List, Union
from fastapi import Query
from obspy import UTCDateTime, Stream
from ... import TimeseriesFactory, TimeseriesUtility
from .DataApiQuery import (
    DEFAULT_ELEMENTS,
    DataHost,
    DataType,
    OutputFormat,
    SamplingPeriod,
)
from .FilterApiQuery import FilterApiQuery


def get_filter_data_query(
    id: str = Query(..., title="Observatory code"),
    starttime: UTCDateTime = Query(
        None,
        title="Start Time",
        description="Time of first requested data. Default is start of current UTC day.",
    ),
    endtime: UTCDateTime = Query(None, title="End Time"),
    elements: List[str] = Query(DEFAULT_ELEMENTS, title="Geomagnetic Elements"),
    data_type: Union[DataType, str] = Query(DataType.VARIATION, alias="type"),
    format: Union[OutputFormat, str] = Query(
        OutputFormat.IAGA2002, title="Output Format"
    ),
    input_sampling_period: Union[SamplingPeriod, float] = Query(
        SamplingPeriod.SECOND, title="Initial sampling period"
    ),
    output_sampling_period: Union[SamplingPeriod, float] = Query(
        SamplingPeriod.MINUTE, title="Output sampling period"
    ),
    data_host: Union[DataHost, str] = Query(
        DataHost.DEFAULT, title="Data Host", description="Edge host to pull data from."
    ),
) -> FilterApiQuery:

    return FilterApiQuery(
        id=id,
        starttime=starttime,
        endtime=endtime,
        elements=elements,
        input_sampling_period=input_sampling_period,
        output_sampling_period=output_sampling_period,
        data_type=data_type,
        data_host=data_host,
        format=format,
    )


def get_timeseries(data_factory: TimeseriesFactory, query: FilterApiQuery) -> Stream:
    """Get timeseries data for variometers

    Parameters
    ----------
    data_factory: where to read data
    query: parameters for the data to read
    """
    # get data
    timeseries = data_factory.get_timeseries(
        starttime=query.starttime,
        endtime=query.endtime,
        observatory=query.id,
        channels=query.elements,
        type=query.data_type,
        interval=TimeseriesUtility.get_interval_from_delta(query.input_sampling_period),
    )
    return timeseries
