from typing import List, Tuple, Optional

from obspy import Trace, UTCDateTime, Stream
from obspy.clients.iris.client import Client
from obspy.clients.fdsn import Client as FDSNClient
from ..geomag_types import DataInterval, DataType
from geomagio.VariometerMetadata import VariometerMetadata
from .. import TimeseriesUtility
from .IRISSNCL import IRISSNCL
from .MiniSeedFactory import MiniSeedFactory


class IRISFactory(MiniSeedFactory):
    """MiniSeedFactory for IRIS related data.

    Parameters
    ----------
    base_url: str
        iris web service url
    host: str
        a string representing the IP number of the host to connect to
    write_port: int
        the port number the miniseed client writes data
    network: str
        data network
    observatory: str
        observatory id
    channels: array
        list of channels
    type: {"adjusted", "definitive", "quasi-definitive", "variation"}
        data type
    interval: {"tenhertz", "second", "minute", "hour", "day"}
        data interval
    variometerMetadata: VariometerMetadata
        variometer metadata to replace the default ObservatoryMetadata
    locationCode: str
        the location code for the given edge server
    convert_channels: array
        list of channels to convert from volt/bin to nT

    See Also
    --------
    MiniSeedFactory

    Notes
    -----
    This factory is designed to receive data from IRIS and write to edge.
    convert_channels should only be used with tenhertz NT data (U, V, W).
    Non-NT data conversions are automatically handled by the IRIS web service.


    """

    def __init__(
        self,
        base_url: str = "http://service.iris.edu/irisws",
        host: str = "cwbpub.cr.usgs.gov",
        write_port: int = 7974,
        network: str = "NT",
        observatory: Optional[str] = None,
        channels: Optional[List[str]] = None,
        type: Optional[DataType] = None,
        interval: Optional[DataInterval] = None,
        locationCode: Optional[str] = None,
        convert_channels: Optional[List[str]] = None,
        variometerMetadata: Optional[VariometerMetadata] = None,
    ):
        super().__init__(
            host=host,
            write_port=write_port,
            observatory=observatory,
            channels=channels,
            type=type,
            interval=interval,
            observatoryMetadata=variometerMetadata or VariometerMetadata(),
            locationCode=locationCode,
            convert_channels=convert_channels,
        )
        self.base_url = base_url
        self.network = network
        self.client = Client(base_url=base_url)
        self.metaClient = FDSNClient

    def _get_timeseries(
        self,
        starttime: UTCDateTime,
        endtime: UTCDateTime,
        observatory: str,
        channel: str,
        type: DataType,
        interval: DataInterval,
        add_empty_channels: bool = True,
    ) -> Trace:
        """get timeseries data for a single channel

        Parameters
        ----------
        starttime: UTCDateTime
            the starttime of requested data
        endtime: UTCDateTime
            the endtime of requested data
        observatory: str
            observatory id
        channel: str
            channel name
        type: {"adjusted", "definitive", "quasi-definitive", "variation"}
            data type
        interval: {"tenhertz", "second", "minute", "hour", "day"}
            data interval
        add_empty_channels: bool
            if True, returns channels without data as empty traces

        Returns
        -------
        data: Trace
            timeseries trace of the requested channel data
        """
        sncl = IRISSNCL.get_sncl(
            data_type=type,
            element=channel,
            interval=interval,
            station=observatory,
            network=self.network,
            location=self.locationCode,
        )
        filter = ["correct"]
        if sncl.channel[1] in ["E", "Y"]:
            filter = []
        data = self.client.timeseries(
            network=sncl.network,
            station=sncl.station,
            location=sncl.location,
            channel=sncl.channel,
            starttime=starttime,
            endtime=endtime,
            filter=filter,
        )
        data.merge()
        if data.count() == 0 and add_empty_channels:
            data += self._get_empty_trace(
                starttime=starttime,
                endtime=endtime,
                observatory=observatory,
                channel=channel,
                data_type=type,
                interval=interval,
                network=sncl.network,
                location=sncl.location,
            )
        if data.count() != 0:
            TimeseriesUtility.pad_and_trim_trace(
                trace=data[0], starttime=starttime, endtime=endtime
            )
        # Beneath is necessary code to check the reported azimuth
        # for the LF1 (E or Y) and LF2 (H or X) channels and determine if intstrument axes have been reversed.
        azi, dip = self._get_orientations(data[0], starttime)

        # Checks Azimuth for LF2 channel
        if 270 > azi and azi > 90.0 and data[0].stats.channel == "LF2":
            data[0].data *= -1

        # Checks Azimuth for LF1 channel
        elif azi > 180 and azi < 360 and data[0].stats.channel == "LF1":
            data[0].data *= -1

        # Checks Dip for LFZ channel
        elif dip > 0 and dip < 180 and data[0].stats.channel == "LFZ":
            data[0].data *= -1

        self._set_metadata(data, observatory, channel, type, interval)
        return data

    # Beneath is necessary code to print azimuth from metadata.
    # This needs to be adjusted so that metadata is returned and if the azimuth is not what is expected(i.e. LF1=0, LF2= 90, and LFZ = -180), the metedata is corrected.

    def _post_process(
        self,
        timeseries: Stream,
        starttime: UTCDateTime,
        endtime: UTCDateTime,
        channels: List[str],
    ):
        """Post process a timeseries stream after the raw data is
                is fetched from querymom. Specifically changes
                any MaskedArray to a ndarray with nans representing gaps.
                Then calls pad_timeseries to deal with gaps at the
                beggining or end of the streams.

        Parameters
        ----------
        timeseries: Stream
            The timeseries stream as returned by the call to get_waveforms
        starttime: UTCDateTime
            the starttime of the requested data
        endtime: UTCDateTime
            the endtime of the requested data
        channels: array
            list of channels to load

        Notes: the original timeseries object is changed.
        """

        TimeseriesUtility.pad_timeseries(timeseries, starttime, endtime)
        # Retrieve and set metadata for each trace
        for trace in timeseries:
            # self._set_metadata(trace, trace.stats.network, trace.stats.station, self.type, self.interval)

            # Retrieve orientation information using FDSN for each trace
            # azimuth = self._get_azimuth_from_orientations(trace, starttime)
            # trace.stats.azimuth = azimuth
            if "comments" in trace.stats:
                trace.stats.comments.extend(
                    [
                        "DISCLAIMER",
                        "This is provisional data. Any data secured from the USGS database that are identified as provisional have not received Director's approval and are subject to revision. Source and Authority: U.S. Geological Survey Manual, Section 500.24",
                        "DISCLAIMER",
                    ]
                )  # input string
            else:
                trace.stats.comments = [
                    "DISCLAIMER",
                    "This is provisional data. Any data secured from the USGS database that are identified as provisional have not received Director's approval and are subject to revision. Source and Authority: U.S. Geological Survey Manual, Section 500.24",
                    "DISCLAIMER",
                ]

    ###################################

    def _get_orientations(
        self, trace: Trace, starttime: UTCDateTime
    ) -> Tuple[float, float]:
        # Retrieve station orientation information using FDSN for each trace
        # print(trace.stats.location)
        sncl = IRISSNCL.get_sncl(
            data_type=trace.stats.location,
            element=trace.stats.channel,
            interval=self.interval,
            station=trace.stats.station,
            network=trace.stats.network,
            location=trace.stats.location,
        )

        FDSN = FDSNClient("IRIS")
        inv = FDSN.get_stations(
            network=sncl.network,
            station=sncl.station,
            channel=sncl.channel,
            level="channel",
        )

        # Construct the channel code using the current trace's information
        channel_code = f"{sncl.network}.{sncl.station}.{sncl.location}.{sncl.channel}"

        # Get orientation for the constructed channel code and time
        orient = inv.get_orientation(channel_code, starttime)

        azimuth = orient["azimuth"]
        dip = orient["dip"]

        # Return both azimuth and dip
        return azimuth, dip

        return 0.0  # Default azimuth if metadata is not available
