from pydantic import BaseModel, Field

from enum import Enum


class InstrumentCategory(str, Enum):
    OFFSETS = "OFFSETS"
    ELECTRONICS = "ELECTRONICS"
    SENSOR = "SENSOR"
    DIGITIZER = "DIGITIZER"
    DATALOGGER = "DATALOGGER"
    THEODOLITE = "THEODOLITE"
    OTHER = "OTHER"


class Instrument(BaseModel):
    """Class for Instrument metadata.

    Attributes
    ----------
    instrument_category:InstrumentCategory
    serial: serial number of instrument.
    model: model number of instrument.
    description: description of instrument.
    """

    instrument_category: InstrumentCategory = "OTHER"
    serial: str = None
    model: str = None
    description: str = None


class Offsets(Instrument):
    """This class is used to store nT offsets. Typically used for FGE offsets.

    Attributes
    ----------
    u_offset: U channel offset in nT.
    v_offset: V channel offset in nT.
    w_offset: W channel offset in nT.
    """

    u_offset: float = Field(..., description="U offset in nT")
    v_offset: float = Field(..., description="V offset in nT")
    w_offset: float = Field(..., description="W offset in nT")

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.instrument_category = "OFFSETS"


class Electronics(Instrument):
    """This class contains sensor electronics information.

    Attributes
    ----------
    type: type of electronics.
    resistance: resistance in kOhm.
    temperature_scale: temperature scale in V/K.
    u_bin: U channel bin number in nT/bin.
    v_bin: V channel bin number in nT/bin.
    w_bin: W channel bin number in nT/bin.
    """

    type: str
    resistance: float = Field(
        default=None, description="scaling resistor values in kOhm"
    )
    temperature_scale: float = Field(
        default=None, description="temperature scale in V/K"
    )
    u_bin: float = Field(default=None, description="U nT/bin")
    v_bin: float = Field(default=None, description="V nT/bin")
    w_bin: float = Field(default=None, description="W nT/bin")

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.instrument_category = "ELECTRONICS"


class Sensor(Instrument):
    """This class contains sensor type and calibration information.

    Attributes
    ----------
    type: type of sensor.
    u_constant: U channel constant in nT/mA.
    v_constant: V channel constant in nT/mA.
    w_constant: W channel constant in nT/mA.
    """

    type: str
    u_constant: float = Field(..., description="U constant in nT/mA")
    v_constant: float = Field(..., description="V constant in nT/mA")
    w_constant: float = Field(..., description="W constant in nT/mA")

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.instrument_category = "SENSOR"


class DataLogger(Instrument):
    """This class contains datalogger information.

    Attributes
    ----------
    type: type of datalogger.
    """

    type: str

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.instrument_category = "DATALOGGER"


class Digitizer(Instrument):
    """This class contains digitizer type and factors. Typically these are from CompactRIO modules.

    Attributes
    ----------
    type: type of digitizer.
    u_scale: U channel scale in counts/V.
    u_offset: U channel offset in counts.
    v_scale: V channel scale in counts/V.
    v_offset: V channel offset in counts.
    w_scale: W channel scale in counts/V.
    w_offset: W channel offset in counts.
    """

    type: str
    u_scale: int = Field(..., description="U scale in counts/V")
    u_offset: int = Field(..., description="U offset in counts")
    v_scale: int = Field(..., description="V scale in counts/V")
    v_offset: int = Field(..., description="V offset in counts")
    w_scale: int = Field(..., description="W scale in counts/V")
    w_offset: int = Field(..., description="W offset in counts")

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.instrument_category = "DIGITIZER"


class Active_Instruments(BaseModel):
    """This class is meant to contain active instruments for a particular observatory.

    It is often referred to as a "bucket".

    Attributes
    ----------
    offsets: offsets metadata.
    electronics: electronics metadata.
    sensor: sensor metadata.
    data_logger: dataLogger metadata.
    digitizer: digitizer metadata.
    """

    offsets: Offsets
    electronics: Electronics
    sensor: Sensor
    data_logger: DataLogger
    digitizer: Digitizer


# Example usage:
if __name__ == "__main__":
    instrument_data = {
        "offsets": {
            "model": None,
            "description": "Observatory offsets",
            "u_offset": 12620,  # nT
            "v_offset": 0,  # nT
            "w_offset": 54907,  # nT
        },
        "electronics": {
            "serial": "E0568",
            "model": "K2",
            "description": "FGE electronics box",
            "type": "FGE Electronics",
            "resistance": 18.6,  # kOhm
            "temperature_scale": 0.01,  # V/K
            "u_bin": None,  # nT/bin
            "v_bin": None,  # nT/bin
            "w_bin": None,  # nT/bin
        },
        "sensor": {
            "serial": "S0443",
            "model": None,
            "description": "sensor constants combine with electronics resistance to give scale values",
            "type": "FGE",
            "u_constant": 37062,  # nT/mA
            "v_constant": 37141,  # nT/mA
            "w_constant": 37281,  # nT/mA
        },
        "data_logger": {
            "serial": "S0443",
            "model": None,
            "description": None,
            "type": "ObsRIO Chassis",
        },
        "digitizer": {
            "serial": "1EA7C53",
            "model": None,
            "description": None,
            "type": "ObsRIO Module",
            "u_scale": 37062,  # counts/V
            "v_scale": 37141,  # counts/V
            "w_scale": 37281,  # counts/V
        },
    }

    instruments = Active_Instruments(**instrument_data)
    print(instruments)
