from obspy import UTCDateTime

from requests.exceptions import JSONDecodeError, ConnectTimeout

from geomagio.metadata import Metadata, MetadataFactory, MetadataCategory
from geomagio.Util import write_state_file, read_state_file


class InstrumentCalibrations:
    """
    A class used to get calibrations from instrument metadata.

    ...

    Attributes
    ----------
        metadata_list : list
            a list of metadata objects to be processed
        previous_calibration : dict
            a dictionary to store the previous calibration values for each axis and key
        station : str
            the station code, checked for consistency across all metadata objects
        location : str
            the location code, checked for consistency across all metadata objects
        network : str
            the network code, checked for consistency across all metadata objects

    Methods
    -------
        get_calibrations():
            Main method to gather applicable calibrations from the metadata list.
        create_overlap_interval(current_metadata):
            Creates an overlap interval from the current metadata.
        update_overlap_interval(overlap_interval, current_metadata):
            Updates the overlap interval with the current metadata.
        update_overlap_interval_with_same_starttime(i, sorted_metadata_list, overlap_interval):
            Updates the overlap interval with metadata that have the same starttime.
        set_endtime(i, sorted_metadata_list, overlap_interval, current_metadata):
            Sets the endtime for the overlap interval.
        convert_to_calibration(overlap_metadata):
            Converts overlapping metadata information to an applicable calibration.
        get_channels(overlap_metadata):
            Gets the channels for the applicable calibration.
    """

    def __init__(self, metadata_list):
        """
        Constructs necessary attributes for the InstrumentCalibrations object and verify consistency of metadata_list.

        Parameters
        ----------
            metadata_list : list
                a list of metadata objects to be processed
        """
        self.metadata_list = metadata_list
        self.previous_calibration = {
            f"{axis}_{key}": None
            for axis in ["u", "v", "w"]
            for key in ["constant", "bin", "offset"]
        }
        self.previous_calibration.update(
            {"resistance": None, "temperature_scale": None, "type": None}
        )

        # Check for consistency in "station", "location", and "network" codes
        stations = {metadata.station for metadata in metadata_list}
        locations = {metadata.location for metadata in metadata_list}
        networks = {metadata.network for metadata in metadata_list}

        if len(stations) > 1:  # or len(locations) > 1:
            raise ValueError(
                f"Inconsistent 'station' codes in metadata_list: {stations}"
            )
        else:
            self.station = stations.pop()
            self.location = locations.pop()
            self.network = networks.pop()

    def get_calibrations(self):
        """
        Main method to compile applicable calibrations from the metadata list.

        Parameters
        ----------
            query_starttime : UTCDateTime
                the query starttime

        Returns
        -------
            list
                a list of dictionaries, each representing an applicable calibration
        """
        sorted_metadata_list = sorted(
            self.metadata_list,
            key=lambda x: x.starttime if x.starttime is not None else UTCDateTime(0),
        )
        i = 0
        calibrations = []
        while i < len(sorted_metadata_list):
            current_metadata = sorted_metadata_list[i]
            overlap_interval = self.create_overlap_interval(current_metadata)
            i = self.update_overlap_interval_with_same_starttime(
                i, sorted_metadata_list, overlap_interval
            )
            self.set_endtime(
                i, sorted_metadata_list, overlap_interval, current_metadata
            )
            calibration = self.convert_to_calibration(overlap_interval)
            # if (
            #     calibration["end_time"] >= query_starttime
            #     or calibration["end_time"] == None
            # ):
            calibrations.append(calibration)
            i += 1
        return calibrations

    def create_overlap_interval(self, current_metadata):
        """
        Creates an overlap interval from the current metadata.

        Parameters
        ----------
            current_metadata : Metadata
                the current metadata object

        Returns
        -------
            dict
                a dictionary representing an overlap interval
        """
        overlap_interval = {"starttime": current_metadata.starttime}
        self.update_overlap_interval(overlap_interval, current_metadata)
        return overlap_interval

    def update_overlap_interval(self, overlap_interval, current_metadata):
        """
        Updates the overlap interval with the current metadata.

        Parameters
        ----------
            overlap_interval : dict
                the overlap interval to be updated
            current_metadata : Metadata
                the current metadata object
        """
        for axis in ["u", "v", "w"]:
            for key in ["constant", "bin", "offset"]:
                current_key = f"{axis}_{key}"
                if current_key in current_metadata.metadata:
                    overlap_interval[current_key] = current_metadata.metadata[
                        current_key
                    ]
                    self.previous_calibration[current_key] = current_metadata.metadata[
                        current_key
                    ]
                elif current_key in self.previous_calibration:
                    overlap_interval[current_key] = self.previous_calibration[
                        current_key
                    ]
        for key in ["resistance", "temperature_scale", "type"]:
            if key in current_metadata.metadata:
                overlap_interval[key] = current_metadata.metadata[key]
                self.previous_calibration[key] = current_metadata.metadata[key]
            elif key in self.previous_calibration:
                overlap_interval[key] = self.previous_calibration[key]

    def update_overlap_interval_with_same_starttime(
        self, i, sorted_metadata_list, overlap_interval
    ):
        """
        Updates the overlap interval with metadata that have the same starttime.

        Parameters
        ----------
            i : int
                the current index in the sorted metadata list
            sorted_metadata_list : list
                the sorted list of metadata objects
            overlap_interval : dict
                the overlap interval to be updated

        Returns
        -------
            int
                the updated index in the sorted metadata list
        """
        while (
            i + 1 < len(sorted_metadata_list)
            and sorted_metadata_list[i + 1].starttime
            == sorted_metadata_list[i].starttime
        ):
            i += 1
            next_metadata = sorted_metadata_list[i]
            self.update_overlap_interval(overlap_interval, next_metadata)
        return i

    def set_endtime(self, i, sorted_metadata_list, overlap_interval, current_metadata):
        """
        Sets the endtime for the overlap interval.

        Parameters
        ----------
            i : int
                the current index in the sorted metadata list
            sorted_metadata_list : list
                the sorted list of metadata objects
            overlap_interval : dict
                the overlap interval to be updated
            current_metadata : Metadata
                the current metadata object
        """
        if (
            i + 1 < len(sorted_metadata_list)
            and sorted_metadata_list[i + 1].starttime != current_metadata.starttime
        ):
            overlap_interval["endtime"] = sorted_metadata_list[i + 1].starttime
        else:
            overlap_interval["endtime"] = current_metadata.endtime

    def convert_to_calibration(self, overlap_metadata):
        """
        Converts an overlap_metadata object to an applicable calibration.

        Parameters
        ----------
            overlap_metadata : dict
                the metadata overlap data to be converted

        Returns
        -------
            dict
                a dictionary representing an applicable calibration
        """
        calibration = {
            "network": self.network,
            "station": self.station,
            "location": self.location,
            "start_time": overlap_metadata["starttime"],
            "end_time": overlap_metadata["endtime"],
            "instrument": {
                "type": overlap_metadata["type"],
                "channels": self.get_channels(overlap_metadata),
            },
        }
        return calibration

    def get_channels(self, overlap_metadata):
        """
        Gets the channels for the applicable calibration.

        Parameters
        ----------
            overlap_metadata : dict
                the metadata overlap data from which to get the channels

        Returns
        -------
            dict
                a dictionary representing the channels for the applicable calibration
        """
        channels = {}
        for axis in ["U", "V", "W"]:
            if (
                f"{axis.lower()}_bin" in overlap_metadata
                and overlap_metadata[f"{axis.lower()}_bin"] is not None
            ):
                channels[axis] = [
                    {
                        "channel": f"{axis}_Volt",
                        "offset": (
                            overlap_metadata[f"{axis.lower()}_offset"]
                            if overlap_metadata[f"{axis.lower()}_offset"] is not None
                            else 0
                        ),
                        "scale": overlap_metadata[f"{axis.lower()}_constant"],
                    },
                    {
                        "channel": f"{axis}_Bin",
                        "offset": 0,
                        "scale": overlap_metadata[f"{axis.lower()}_bin"],
                    },
                ]
            elif (
                "resistance" in overlap_metadata
                and overlap_metadata["resistance"] is not None
            ):
                channels[axis] = [
                    {
                        "channel": f"{axis}_Volt",
                        "offset": (
                            overlap_metadata[f"{axis.lower()}_offset"]
                            if overlap_metadata[f"{axis.lower()}_offset"] is not None
                            else 0
                        ),
                        "scale": overlap_metadata[f"{axis.lower()}_constant"]
                        / overlap_metadata["resistance"],
                    }
                ]
        return channels


def get_instrument_calibrations(
    observatory, start_time=None, end_time=None, calibrations=None, metadata_url=None
):
    """Get instrument metadata

    Parameters:
    -----------
        observatory: observatory code
        start_time: start time to match, or None to match any.
        end_time: end time to match, or None to match any.
        calibrations: use custom list, defaults to pulling and converting instrument metadata
        metadata_url: metadata database url

    Returns:
    --------
        list of applicable instrument calibrations
    """

    if not calibrations:
        state_filename = f"{observatory}_instrument_cals.json"
        metadata = []

        factory = MetadataFactory(
            url=metadata_url or "https://staging-geomag.cr.usgs.gov/ws/secure/metadata",
        )
        query = Metadata(
            category=MetadataCategory.INSTRUMENT,
            starttime=start_time,
            endtime=end_time,
            station=observatory,
            data_valid=True,
        )
        try:
            metadata = factory.get_metadata(query=query, timeout=28)
        except:
            print(
                "Warning: An error occurred while trying to pull metadata from the metadata server!"
            )

        if not metadata:
            print(
                f"Warning: No valid metadata returned for {observatory} for time interval: {start_time} - {end_time}"
            )
        else:
            try:
                instrumentCalibrations = InstrumentCalibrations(metadata)
                calibrations = instrumentCalibrations.get_calibrations()
            except ValueError as e:
                print(e)

        if not calibrations:
            print(
                f"Warning: No valid calibrations generated for {observatory} for time interval: {start_time} - {end_time}"
            )
            print(f"Pulling calibrations for {observatory} from state file.")

            calibrations = read_state_file(filename=state_filename)
        else:
            try:
                write_state_file(state_filename, calibrations)
            except TypeError:
                print("Write state file failed, object couldn't be encoded.")

    return [
        c
        for c in calibrations
        if c["station"] == observatory
        and (end_time is None or c["start_time"] is None or c["start_time"] < end_time)
        and (start_time is None or c["end_time"] is None or c["end_time"] > start_time)
    ]
